//+----------------------------------------------------------------------------+
//|                                                                Locale.mqh  |
//|                                                     Kim Igor . aka KimIV  |
//|                                                       http://www.kimiv.ru  |
//|                                                                            |
//|  07.04.2007  The Library of functions for work with regional settings.     |
//+----------------------------------------------------------------------------+

#import "kernel32.dll"
  int GetSystemDefaultLCID();
  int GetUserDefaultLCID();
  int GetLocaleInfoA(
    int    Locale,      // locale identifier
    int    LCType,      // type of information
    string lpLCData,    // address of buffer for information
    int    cchData      // size of buffer
  );
#import

// LCTYPE Constants
#define LOCALE_ILANGUAGE              0x0001 // A LANGID represented in hexadecimal digits.
#define LOCALE_SLANGUAGE              0x0002 // The full localized name of the language.
#define LOCALE_SENGLANGUAGE           0x1001 // The full English U.S. name of the
                                             // language from the ISO Standard 639.
                                             // This will always be restricted to
                                             // characters that can be mapped into
                                             // the ASCII 127-character subset.
#define LOCALE_SABBREVLANGNAME        0x0003 // The abbreviated name of the language,
                                             // created by taking the two-letter
                                             // language abbreviation, as found in
                                             // ISO Standard 639, and adding a third
                                             // letter as appropriate to indicate
                                             // the sublanguage.
#define LOCALE_SNATIVELANGNAME        0x0004 // The native name of the language.
#define LOCALE_ICOUNTRY               0x0005 // The country code, based on international phone codes, also referred to as IBM country codes.
#define LOCALE_SCOUNTRY               0x0006 // The full localized name of the country.
#define LOCALE_SENGCOUNTRY            0x1002 // The full English U.S. name of the country. This will always be restricted to characters that can be mapped into the ASCII 127-character subset.
#define LOCALE_SABBREVCTRYNAME        0x0007 // The abbreviated name of the country as found in ISO Standard 3166.
#define LOCALE_SNATIVECTRYNAME        0x0008 // The native name of the country.
#define LOCALE_IDEFAULTLANGUAGE       0x0009 // LANGID for the principal language spoken in this locale. This is provided so that partially specified locales can be completed with default values.
#define LOCALE_IDEFAULTCOUNTRY        0x000A // Country code for the principal country in this locale. This is provided so that partially specified locales can be completed with default values.
//#define LOCALE_IDEFAULTANSICODEPAGE   // American National Standards Institute (ANSI) code page associated with this locale. The maximum number of characters allowed for this string is 6.
//#define LOCALE_IDEFAULTOEMCODEPAGE    // Original equipment manufacturer (OEM) code page associated with the locale. The maximum number of characters allowed for this string is 6.
#define LOCALE_IDEFAULTCODEPAGE       0x000B // The OEM code page associated with the country.
//#define LOCALE_IDEFAULTEBCDICCODEPAGE // Default EBCDIC code page associated with the locale. The maximum number of characters allowed for this string is 6.
#define LOCALE_SLIST                  0x000C // Characters used to separate list items. For example, a comma is used in many locales.
#define LOCALE_IMEASURE               0x000D // This value is 0 for the metric system (S.I.) and 1 for the U.S. system of measurements.
#define LOCALE_SDECIMAL               0x000E // Characters used for the decimal separator.
#define LOCALE_STHOUSAND              0x000F // Characters used as the separator between groups of digits left of the decimal.
#define LOCALE_SGROUPING              0x0010 // Sizes for each group of digits to the left of the decimal. An explicit size is required for each group. Sizes are separated by semicolons. If the last value is 0, the preceding value is repeated. To group thousands, specify 3;0.
#define LOCALE_IDIGITS                0x0011 // The number of fractional digits.
#define LOCALE_ILZERO                 0x0012 // Specifier for leading zeros in
                                             // decimal fields. The maximum number
                                             // of characters allowed for this
                                             // string is 2. The specifier can be
                                             // one of the following values:
                                             //   0 No leading zeros
                                             //   1 Leading zeros
//#define LOCALE_INEGNUMBER // Negative number mode. The mode can be one of these values:
//0 (1.1)
//1 1.1
//2  1.1
//3 1.1
//4 1.1 
#define LOCALE_SNATIVEDIGITS   0x0013 // Native equivalents to ASCII 0 through 9.
#define LOCALE_SCURRENCY       0x0014 // String used as the local monetary symbol.
//#define LOCALE_SENGCURRNAME    // The full English name of the currency associated with the locale.
//#define LOCALE_SNATIVECURRNAME // The native name of the currency associated with the locale.
#define LOCALE_SINTLSYMBOL     0x0015 // Three characters of the international monetary symbol specified in ISO 4217, "Codes for the Representation of Currencies and Funds," followed by the character separating this string from the amount.
#define LOCALE_SMONDECIMALSEP  0x0016 // Character(s) used as the monetary decimal separator.
#define LOCALE_SMONTHOUSANDSEP 0x0017 // Character(s) used as the monetary separator between groups of digits to the left of the decimal.
#define LOCALE_SMONGROUPING    0x0018 // Sizes for each group of monetary digits to the left of the decimal. An explicit size is needed for each group; sizes are separated by semicolons. If the last value is zero, the preceding value is repeated. To group thousands, specify 3;0, for example.
#define LOCALE_ICURRDIGITS     0x0019 // Number of fractional digits for the local monetary format. The maximum number of characters allowed for this string is 3.
#define LOCALE_IINTLCURRDIGITS 0x001A // Number of fractional digits for the international monetary format. The maximum number of characters allowed for this string is 3.
#define LOCALE_ICURRENCY       0x001B // Positive currency mode. The maximum number
                                      // of characters allowed for this string is 2.
                                      // The mode can be one of the following values:
                                      //   0 Prefix, no separation
                                      //   1 Suffix, no separation
                                      //   2 Prefix, 1-char. separation
                                      //   3 Suffix, 1-char. separation
#define LOCALE_INEGCURR 0x001C    // Negative currency mode. The maximum number
                                  // of characters allowed for this string is 3.
                                  // The mode can be one of the following values:
                                  //    0 ($1.1)
                                  //    1 $1.1
                                  //    2 $1.1
                                  //    3 $1.1
                                  //    4 (1.1$)
                                  //    5 1.1$
                                  //    6 1.1$
                                  //    7 1.1$
                                  //    8 1.1 $ (space before $)
                                  //    9 $ 1.1 (space after $)
                                  //   10 1.1 $ (space before $)
                                  //   11 $ 1.1 (space after $)
                                  //   12 $ 1.1 (space after $)
                                  //   13 1.1 $ (space before $)
                                  //   14 ($ 1.1) (space after $)
                                  //   15 (1.1 $) (space before $)
#define LOCALE_SDATE       0x001D // Character(s) for the date separator.
#define LOCALE_STIME       0x001E // Character(s) for the time separator.
#define LOCALE_SSHORTDATE  0x001F // Short date formatting string for this locale. The string can consist of a combination of day, month, and year format pictures defined in Day, Month, Year, and Era Format Pictures table in National Language Support Constants.
#define LOCALE_SLONGDATE   0x0020 // Long date formatting string for this locale. The string can consist of a combination of day, month, and year format pictures defined in the Day, Month, Year, and Era Format Pictures table in National Language Support Constants and any string of characters enclosed in single quotes. Characters in single quotes remain as given.
#define LOCALE_STIMEFORMAT 0x1003 // Time formatting strings for this locale. The string can consist of a combination of the hour, minute, and second format pictures defined in the Hour, Minute, and Second Format Pictures table in National Language Support Constants.
//#define LOCALE_SYEARMONTH  // The Year/Month formatting string for the locale. This string shows the proper format for a date string that contains only the year and the month.
#define LOCALE_IDATE       0x0021 // Short date format-ordering specifier.
                                  // The maximum number of characters allowed
                                  // for this string is 2. The specifier can be
                                  // one of the following values:
                                  //   0 Month-Day-Year
                                  //   1 Day-Month-Year
                                  //   2 Year-Month-Day
#define LOCALE_ILDATE      0x0022 // Long date format-ordering specifier.
                                  // The maximum number of characters allowed for
                                  // this string is 2. The specifier can be one
                                  // of the following values:
                                  //   0 Month-Day-Year
                                  //   1 Day-Month-Year
                                  //   2 Year-Month-Day
#define LOCALE_ITIME       0x0023 // Time format specifier. The maximum number
                                  // of characters allowed for this string is 2.
                                  // The specifier can be one of the following values:
                                  //   0 AM / PM 12-hour format
                                  //   1 24-hour format
#define LOCALE_ICENTURY    0x0024 // Specifier for full 4-digit century.
                                  // The maximum number of characters allowed for
                                  // this string is 2. The specifier can be one of
                                  // the following values:
                                  //   0 Abbreviated 2-digit century
                                  //   1 Full 4-digit century
#define LOCALE_ITLZERO     0x0025 // Specifier for leading zeros in time fields.
                                  // The maximum number of characters allowed for
                                  // this string is 2. The specifier can be one
                                  // of the following values:
                                  //   0 No leading zeros for hours
                                  //   1 Leading zeros for hours
#define LOCALE_IDAYLZERO   0x0026 // Specifier for leading zeros in day fields.
                                  // The maximum number of characters allowed for
                                  // this string is 2. The specifier can be one
                                  // of the following values:
                                  //   0 No leading zeros for days
                                  //   1 Leading zeros for days
#define LOCALE_IMONLZERO   0x0027 // Specifier for leading zeros in month fields.
                                  // The maximum number of characters allowed for
                                  // this string is 2. The specifier can be one of
                                  // the following values:
                                  //   0 No leading zeros for months
                                  //   1 Leading zeros for months
#define LOCALE_S1159       0x0028 // String for the AM designator.
#define LOCALE_S2359       0x0029 // String for the PM designator.
//#define LOCALE_ICALENDARTYPE // Current calendar type. This type can be one of these values:
//1 Gregorian (as in United States)
//2 Gregorian (English strings always)
//3 Era: Year of the Emperor (Japan)
//4 Era: Year of Taiwan Region
//5 Tangun Era (Korea)
//#define LOCALE_IOPTIONALCALENDAR // Additional calendar types. This can be a zero-separated list of one or more of these calendars type values:
//0 No additional types valid
//1 Gregorian (as in United States)
//2 Gregorian (English strings always)
//3 Era: Year of the Emperor (Japan)
//4 Era: Year of Taiwan Region
//5 Tangun Era (Korea)
//#define LOCALE_IFIRSTDAYOFWEEK // Specifier for the first day in a week. The specifier can be one of these values:
//0 LOCALE_SDAYNAME1
//1 LOCALE_SDAYNAME2
//2 LOCALE_SDAYNAME3
//3 LOCALE_SDAYNAME4
//4 LOCALE_SDAYNAME5
//5 LOCALE_SDAYNAME6
//6 LOCALE_SDAYNAME7
//#define LOCALE_IFIRSTWEEKOFYEAR // Specifier for the first week of the year. The specifier can be one of these values:
//0 Week containing 1/1 is the first week of that year.
//1 First full week following 1/1 is the first week of that year.
//2 First week containing at least 4 days is the first week of that year.
#define LOCALE_SDAYNAME1          0x002A // Native long name for Monday.
#define LOCALE_SDAYNAME2          0x002B // Native long name for Tuesday.
#define LOCALE_SDAYNAME3          0x002C // Native long name for Wednesday.
#define LOCALE_SDAYNAME4          0x002D // Native long name for Thursday.
#define LOCALE_SDAYNAME5          0x002E // Native long name for Friday.
#define LOCALE_SDAYNAME6          0x002F // Native long name for Saturday.
#define LOCALE_SDAYNAME7          0x0030 // Native long name for Sunday.
#define LOCALE_SABBREVDAYNAME1    0x0031 // Native abbreviated name for Monday.
#define LOCALE_SABBREVDAYNAME2    0x0032 // Native abbreviated name for Tuesday.
#define LOCALE_SABBREVDAYNAME3    0x0033 // Native abbreviated name for Wednesday.
#define LOCALE_SABBREVDAYNAME4    0x0034 // Native abbreviated name for Thursday.
#define LOCALE_SABBREVDAYNAME5    0x0035 // Native abbreviated name for Friday.
#define LOCALE_SABBREVDAYNAME6    0x0036 // Native abbreviated name for Saturday.
#define LOCALE_SABBREVDAYNAME7    0x0037 // Native abbreviated name for Sunday.
#define LOCALE_SMONTHNAME1        0x0038 // Native long name for January.
#define LOCALE_SMONTHNAME2        0x0039 // Native long name for February.
#define LOCALE_SMONTHNAME3        0x003A // Native long name for March.
#define LOCALE_SMONTHNAME4        0x003B // Native long name for April.
#define LOCALE_SMONTHNAME5        0x003C // Native long name for May.
#define LOCALE_SMONTHNAME6        0x003D // Native long name for June.
#define LOCALE_SMONTHNAME7        0x003E // Native long name for July.
#define LOCALE_SMONTHNAME8        0x003F // Native long name for August.
#define LOCALE_SMONTHNAME9        0x0040 // Native long name for September.
#define LOCALE_SMONTHNAME10       0x0041 // Native long name for October.
#define LOCALE_SMONTHNAME11       0x0042 // Native long name for November.
#define LOCALE_SMONTHNAME12       0x0043 // Native long name for December.
//#define LOCALE_SMONTHNAME13
#define LOCALE_SABBREVMONTHNAME1  0x0044 // Native abbreviated name for January.
#define LOCALE_SABBREVMONTHNAME2  0x0045 // Native abbreviated name for February.
#define LOCALE_SABBREVMONTHNAME3  0x0046 // Native abbreviated name for March.
#define LOCALE_SABBREVMONTHNAME4  0x0047 // Native abbreviated name for April.
#define LOCALE_SABBREVMONTHNAME5  0x0048 // Native abbreviated name for May.
#define LOCALE_SABBREVMONTHNAME6  0x0049 // Native abbreviated name for June.
#define LOCALE_SABBREVMONTHNAME7  0x004A // Native abbreviated name for July.
#define LOCALE_SABBREVMONTHNAME8  0x004B // Native abbreviated name for August.
#define LOCALE_SABBREVMONTHNAME9  0x004C // Native abbreviated name for September.
#define LOCALE_SABBREVMONTHNAME10 0x004D // Native abbreviated name for October.
#define LOCALE_SABBREVMONTHNAME11 0x004E // Native abbreviated name for November.
#define LOCALE_SABBREVMONTHNAME12 0x004F // Native abbreviated name for December.
#define LOCALE_SABBREVMONTHNAME13 0x100F // Native abbreviated name for 13th month, if it exists.
//#define LOCALE_SPOSITIVESIGN      String value for the positive sign.
//#define LOCALE_SNEGATIVESIGN      String value for the negative sign.
//#define LOCALE_IPOSSIGNPOSN       Formatting index for positive values. The maximum number of characters allowed for this string is 2. The index can be one of the following values:
//0 Parentheses surround the amount and the monetary symbol.
//1 The sign string precedes the amount and the monetary symbol.
//2 The sign string succeeds the amount and the monetary symbol.
//3 The sign string immediately precedes the monetary symbol.
//4 The sign string immediately succeeds the monetary symbol.
//#define LOCALE_INEGSIGNPOSN    Formatting index for negative values. This index uses the same values as LOCALE_IPOSSIGNPOSN. The maximum number of characters allowed for this string is 2.
//#define LOCALE_IPOSSYMPRECEDES Position of monetary symbol in a positive monetary value. This value is 1 if the monetary symbol precedes the positive amount, 0 if it follows it. The maximum number of characters allowed for this string is 2.
//#define LOCALE_IPOSSEPBYSPACE  Separation of monetary symbol in a positive monetary value. This value is 1 if the monetary symbol is separated by a space from a positive amount, 0 if it is not. The maximum number of characters allowed for this string is 2.
//#define LOCALE_INEGSYMPRECEDES Position of monetary symbol in a negative monetary value. This value is 1 if the monetary symbol precedes the negative amount, 0 if it follows it. The maximum number of characters allowed for this string is 2.
//#define LOCALE_INEGSEPBYSPACE  Separation of monetary symbol in a negative monetary value. This value is 1 if the monetary symbol is separated by a space from the negative amount, 0 if it is not. The maximum number of characters allowed for this string is 2.
//#define LOCALE_IPAPERSIZE      Default paper size associated with the locale.
//#define LOCALE_NOUSEROVERRIDE  This constant may be OR'ed with any other LCTYPE constant in a call to the GetLocaleInfo function. This always causes the function to bypass any user overrides, and return the system default value for the other LCTYPE specified in the function call, based on the given LCID.


//+----------------------------------------------------------------------------+
//|                   |
//|      .                                      |
//|  :                                                                |
//|    v -   double                                                   |
//|    d -                                         |
//+----------------------------------------------------------------------------+
string DoubleToStrLocale(double v, int d) {
  if (d<0) d=0;
  if (d>8) d=8;

  string se=GetUserLocale(LOCALE_SDECIMAL), st=DoubleToStr(v, d);
  if (se!=".") st=StrTran(st, ".", se);

  return(st);
}

//+----------------------------------------------------------------------------+
//|  Return of system regional settings.                                       |
//|  Parameters:                                                               |
//|    LCType - type of regional setting (LCTYPE Constant)                     |
//+----------------------------------------------------------------------------+
string GetSystemLocale(int LCType) {
  int    loc=GetSystemDefaultLCID();
  string str="                                                                ";

  GetLocaleInfoA(loc, LCType, str, 64);

  return(str);
}

//+----------------------------------------------------------------------------+
//|  Return of user regional settings.                                         |
//|  Parameters:                                                               |
//|    LCType - type of regional setting (LCTYPE Constant)                     |
//+----------------------------------------------------------------------------+
string GetUserLocale(int LCType) {
  int    loc=GetUserDefaultLCID();
  string str="                                                                ";

  GetLocaleInfoA(loc, LCType, str, 64);

  return(str);
}

//+----------------------------------------------------------------------------+
//|                                                             |
//|  :                                                                |
//|    str     -  ,                   |
//|    strfrom -                                            |
//|    strto   -                                            |
//+----------------------------------------------------------------------------+
string StrTran(string str, string strfrom, string strto) {
  int    n;
  string outstr="", tempstr;

  for (n=0; n<StringLen(str); n++) {
    tempstr=StringSubstr(str, n, StringLen(strfrom));
    if (tempstr==strfrom) {
      outstr=outstr+strto;
      n=n+StringLen(strfrom)-1;
    } else outstr=outstr+StringSubstr(str, n, 1);
  }
  return(outstr);
}
//+----------------------------------------------------------------------------+

